'use strict';

/**
 * ON CHANGE
 * Track the value of anything (through a function)
 * and call the callback when it does.
 * It doesn't do deep comparisons.
 * It doesn't verify automatically,
 * you decide when it may be necessary with observer.verify()
 *

EXAMPLE:

const observer = onChange(
	() => valueToTrack,
	(newValue, oldValue) => console.log(oldValue, 'changed to', newValue)
);

observer.verify();
setInterval(observer.verify.bind(observer), 100)

*/

function onChange (getter, callback) {
	var lastValue = getter();
	var currentValue = void 0;
	return {
		verify: function verify() {
			currentValue = getter();
			if (currentValue !== lastValue) {
				callback(currentValue, lastValue);
				lastValue = currentValue;
			}
		}
	};
}

var handlers = [];
var timer = void 0;

function getVisibility() {
	return document.hidden;
}
var observer = onChange(getVisibility, function () {
	return handlers.forEach(function (fn) {
		return setTimeout(fn, 0);
	});
});

function maybeStartLoop() {
	if (!timer) {
		timer = setInterval(observer.verify, 333);
	}
}
function maybeEndLoop() {
	if (!handlers.length) {
		timer = clearInterval(timer);
	}
}
function onVisibilityChange(handler) {
	// Use onChange to make sure that the handler is
	// only called once by either the browser or the ponyfill
	var debouncer = onChange(getVisibility, handler);

	document.addEventListener('visibilitychange', debouncer.verify);
	handlers.push(debouncer.verify);

	maybeStartLoop();
	var remove = function remove() {
		handlers = handlers.filter(function (fn) {
			return fn !== debouncer.verify;
		});
		document.removeEventListener('visibilitychange', debouncer.verify);
		maybeEndLoop();
	};

	// Store the remove function on the handler
	// so it can be called by removeAll()
	debouncer.verify.remove = remove;
	return { remove: remove };
}
onVisibilityChange.removeAll = function () {
	handlers.forEach(function (fn) {
		fn.remove();
	});
	handlers.length = 0;
	maybeEndLoop();
};

module.exports = onVisibilityChange;